@@ -3,20 +3,29 @@
 use core::ops::Deref;
 use std::io;
 
-use crate::format_description::well_known::{Rfc2822, Rfc3339};
-use crate::format_description::FormatItem;
+use crate::format_description::well_known::iso8601::EncodedConfig;
+use crate::format_description::well_known::{Iso8601, Rfc2822, Rfc3339};
+use crate::format_description::{FormatItem, OwnedFormatItem};
 use crate::formatting::{
-    format_component, format_number_pad_zero, write, MONTH_NAMES, WEEKDAY_NAMES,
+    format_component, format_number_pad_zero, iso8601, write, MONTH_NAMES, WEEKDAY_NAMES,
 };
 use crate::{error, Date, Time, UtcOffset};
 
-/// A type that can be formatted.
+/// A type that describes a format.
+///
+/// Implementors of [`Formattable`] are [format descriptions](crate::format_description).
+///
+/// [`Date::format`] and [`Time::format`] each use a format description to generate
+/// a String from their data. See the respective methods for usage examples.
 #[cfg_attr(__time_03_docs, doc(notable_trait))]
 pub trait Formattable: sealed::Sealed {}
 impl Formattable for FormatItem<'_> {}
 impl Formattable for [FormatItem<'_>] {}
+impl Formattable for OwnedFormatItem {}
+impl Formattable for [OwnedFormatItem] {}
 impl Formattable for Rfc3339 {}
 impl Formattable for Rfc2822 {}
+impl<const CONFIG: EncodedConfig> Formattable for Iso8601<CONFIG> {}
 impl<T: Deref> Formattable for T where T::Target: Formattable {}
 
 /// Seal the trait to prevent downstream users from implementing it.
@@ -25,7 +34,6 @@ mod sealed {
     use super::*;
 
     /// Format the item using a format description, the intended output, and the various components.
-    #[cfg_attr(__time_03_docs, doc(cfg(feature = "formatting")))]
     pub trait Sealed {
         /// Format the item into the provided output, returning the number of bytes written.
         fn format_into(
@@ -51,7 +59,7 @@ mod sealed {
 }
 
 // region: custom formats
-impl<'a> sealed::Sealed for FormatItem<'a> {
+impl sealed::Sealed for FormatItem<'_> {
     fn format_into(
         &self,
         output: &mut impl io::Write,
@@ -72,7 +80,44 @@ impl<'a> sealed::Sealed for FormatItem<'a> {
     }
 }
 
-impl<'a> sealed::Sealed for [FormatItem<'a>] {
+impl sealed::Sealed for [FormatItem<'_>] {
+    fn format_into(
+        &self,
+        output: &mut impl io::Write,
+        date: Option<Date>,
+        time: Option<Time>,
+        offset: Option<UtcOffset>,
+    ) -> Result<usize, error::Format> {
+        let mut bytes = 0;
+        for item in self.iter() {
+            bytes += item.format_into(output, date, time, offset)?;
+        }
+        Ok(bytes)
+    }
+}
+
+impl sealed::Sealed for OwnedFormatItem {
+    fn format_into(
+        &self,
+        output: &mut impl io::Write,
+        date: Option<Date>,
+        time: Option<Time>,
+        offset: Option<UtcOffset>,
+    ) -> Result<usize, error::Format> {
+        match self {
+            Self::Literal(literal) => Ok(write(output, literal)?),
+            Self::Component(component) => format_component(output, *component, date, time, offset),
+            Self::Compound(items) => items.format_into(output, date, time, offset),
+            Self::Optional(item) => item.format_into(output, date, time, offset),
+            Self::First(items) => match &**items {
+                [] => Ok(0),
+                [item, ..] => item.format_into(output, date, time, offset),
+            },
+        }
+    }
+}
+
+impl sealed::Sealed for [OwnedFormatItem] {
     fn format_into(
         &self,
         output: &mut impl io::Write,
@@ -133,22 +178,21 @@ impl sealed::Sealed for Rfc2822 {
             &WEEKDAY_NAMES[date.weekday().number_days_from_monday() as usize][..3],
         )?;
         bytes += write(output, b", ")?;
-        bytes += format_number_pad_zero::<_, _, 2>(output, day)?;
+        bytes += format_number_pad_zero::<2>(output, day)?;
         bytes += write(output, b" ")?;
         bytes += write(output, &MONTH_NAMES[month as usize - 1][..3])?;
         bytes += write(output, b" ")?;
-        bytes += format_number_pad_zero::<_, _, 4>(output, year as u32)?;
+        bytes += format_number_pad_zero::<4>(output, year as u32)?;
         bytes += write(output, b" ")?;
-        bytes += format_number_pad_zero::<_, _, 2>(output, time.hour())?;
+        bytes += format_number_pad_zero::<2>(output, time.hour())?;
         bytes += write(output, b":")?;
-        bytes += format_number_pad_zero::<_, _, 2>(output, time.minute())?;
+        bytes += format_number_pad_zero::<2>(output, time.minute())?;
         bytes += write(output, b":")?;
-        bytes += format_number_pad_zero::<_, _, 2>(output, time.second())?;
+        bytes += format_number_pad_zero::<2>(output, time.second())?;
         bytes += write(output, b" ")?;
         bytes += write(output, if offset.is_negative() { b"-" } else { b"+" })?;
-        bytes += format_number_pad_zero::<_, _, 2>(output, offset.whole_hours().unsigned_abs())?;
-        bytes +=
-            format_number_pad_zero::<_, _, 2>(output, offset.minutes_past_hour().unsigned_abs())?;
+        bytes += format_number_pad_zero::<2>(output, offset.whole_hours().unsigned_abs())?;
+        bytes += format_number_pad_zero::<2>(output, offset.minutes_past_hour().unsigned_abs())?;
 
         Ok(bytes)
     }
@@ -173,64 +217,92 @@ impl sealed::Sealed for Rfc3339 {
         if !(0..10_000).contains(&year) {
             return Err(error::Format::InvalidComponent("year"));
         }
+        if offset.whole_hours().unsigned_abs() > 23 {
+            return Err(error::Format::InvalidComponent("offset_hour"));
+        }
         if offset.seconds_past_minute() != 0 {
             return Err(error::Format::InvalidComponent("offset_second"));
         }
 
-        bytes += format_number_pad_zero::<_, _, 4>(output, year as u32)?;
-        bytes += write(output, &[b'-'])?;
-        bytes += format_number_pad_zero::<_, _, 2>(output, date.month() as u8)?;
-        bytes += write(output, &[b'-'])?;
-        bytes += format_number_pad_zero::<_, _, 2>(output, date.day())?;
-        bytes += write(output, &[b'T'])?;
-        bytes += format_number_pad_zero::<_, _, 2>(output, time.hour())?;
-        bytes += write(output, &[b':'])?;
-        bytes += format_number_pad_zero::<_, _, 2>(output, time.minute())?;
-        bytes += write(output, &[b':'])?;
-        bytes += format_number_pad_zero::<_, _, 2>(output, time.second())?;
+        bytes += format_number_pad_zero::<4>(output, year as u32)?;
+        bytes += write(output, b"-")?;
+        bytes += format_number_pad_zero::<2>(output, date.month() as u8)?;
+        bytes += write(output, b"-")?;
+        bytes += format_number_pad_zero::<2>(output, date.day())?;
+        bytes += write(output, b"T")?;
+        bytes += format_number_pad_zero::<2>(output, time.hour())?;
+        bytes += write(output, b":")?;
+        bytes += format_number_pad_zero::<2>(output, time.minute())?;
+        bytes += write(output, b":")?;
+        bytes += format_number_pad_zero::<2>(output, time.second())?;
 
         #[allow(clippy::if_not_else)]
         if time.nanosecond() != 0 {
             let nanos = time.nanosecond();
-            bytes += write(output, &[b'.'])?;
+            bytes += write(output, b".")?;
             bytes += if nanos % 10 != 0 {
-                format_number_pad_zero::<_, _, 9>(output, nanos)
+                format_number_pad_zero::<9>(output, nanos)
             } else if (nanos / 10) % 10 != 0 {
-                format_number_pad_zero::<_, _, 8>(output, nanos / 10)
+                format_number_pad_zero::<8>(output, nanos / 10)
             } else if (nanos / 100) % 10 != 0 {
-                format_number_pad_zero::<_, _, 7>(output, nanos / 100)
+                format_number_pad_zero::<7>(output, nanos / 100)
             } else if (nanos / 1_000) % 10 != 0 {
-                format_number_pad_zero::<_, _, 6>(output, nanos / 1_000)
+                format_number_pad_zero::<6>(output, nanos / 1_000)
             } else if (nanos / 10_000) % 10 != 0 {
-                format_number_pad_zero::<_, _, 5>(output, nanos / 10_000)
+                format_number_pad_zero::<5>(output, nanos / 10_000)
             } else if (nanos / 100_000) % 10 != 0 {
-                format_number_pad_zero::<_, _, 4>(output, nanos / 100_000)
+                format_number_pad_zero::<4>(output, nanos / 100_000)
             } else if (nanos / 1_000_000) % 10 != 0 {
-                format_number_pad_zero::<_, _, 3>(output, nanos / 1_000_000)
+                format_number_pad_zero::<3>(output, nanos / 1_000_000)
             } else if (nanos / 10_000_000) % 10 != 0 {
-                format_number_pad_zero::<_, _, 2>(output, nanos / 10_000_000)
+                format_number_pad_zero::<2>(output, nanos / 10_000_000)
             } else {
-                format_number_pad_zero::<_, _, 1>(output, nanos / 100_000_000)
+                format_number_pad_zero::<1>(output, nanos / 100_000_000)
             }?;
         }
 
         if offset == UtcOffset::UTC {
-            bytes += write(output, &[b'Z'])?;
+            bytes += write(output, b"Z")?;
             return Ok(bytes);
         }
 
-        bytes += write(
-            output,
-            if offset.is_negative() {
-                &[b'-']
-            } else {
-                &[b'+']
-            },
-        )?;
-        bytes += format_number_pad_zero::<_, _, 2>(output, offset.whole_hours().unsigned_abs())?;
-        bytes += write(output, &[b':'])?;
-        bytes +=
-            format_number_pad_zero::<_, _, 2>(output, offset.minutes_past_hour().unsigned_abs())?;
+        bytes += write(output, if offset.is_negative() { b"-" } else { b"+" })?;
+        bytes += format_number_pad_zero::<2>(output, offset.whole_hours().unsigned_abs())?;
+        bytes += write(output, b":")?;
+        bytes += format_number_pad_zero::<2>(output, offset.minutes_past_hour().unsigned_abs())?;
+
+        Ok(bytes)
+    }
+}
+
+impl<const CONFIG: EncodedConfig> sealed::Sealed for Iso8601<CONFIG> {
+    fn format_into(
+        &self,
+        output: &mut impl io::Write,
+        date: Option<Date>,
+        time: Option<Time>,
+        offset: Option<UtcOffset>,
+    ) -> Result<usize, error::Format> {
+        let mut bytes = 0;
+
+        if Self::FORMAT_DATE {
+            let date = date.ok_or(error::Format::InsufficientTypeInformation)?;
+            bytes += iso8601::format_date::<CONFIG>(output, date)?;
+        }
+        if Self::FORMAT_TIME {
+            let time = time.ok_or(error::Format::InsufficientTypeInformation)?;
+            bytes += iso8601::format_time::<CONFIG>(output, time)?;
+        }
+        if Self::FORMAT_OFFSET {
+            let offset = offset.ok_or(error::Format::InsufficientTypeInformation)?;
+            bytes += iso8601::format_offset::<CONFIG>(output, offset)?;
+        }
+
+        if bytes == 0 {
+            // The only reason there would be no bytes written is if the format was only for
+            // parsing.
+            panic!("attempted to format a parsing-only format description");
+        }
 
         Ok(bytes)
     }
