@@ -1,12 +1,15 @@
 //! Formatting for various types.
 
 pub(crate) mod formattable;
-
+mod iso8601;
+use core::num::NonZeroU8;
 use std::io;
 
 pub use self::formattable::Formattable;
+use crate::convert::*;
+use crate::ext::DigitCount;
 use crate::format_description::{modifier, Component};
-use crate::{error, Date, Time, UtcOffset};
+use crate::{error, Date, OffsetDateTime, Time, UtcOffset};
 
 #[allow(clippy::missing_docs_in_private_items)]
 const MONTH_NAMES: [&[u8]; 12] = [
@@ -35,121 +38,78 @@ const WEEKDAY_NAMES: [&[u8]; 7] = [
     b"Sunday",
 ];
 
-// region: extension trait
-/// A trait that indicates the formatted width of the value can be determined.
-///
-/// Note that this should not be implemented for any signed integers. This forces the caller to
-/// write the sign if desired.
-pub(crate) trait DigitCount {
-    /// The number of digits in the stringified value.
-    fn num_digits(self) -> u8;
-}
-impl DigitCount for u8 {
-    fn num_digits(self) -> u8 {
-        // Using a lookup table as with u32 is *not* faster in standalone benchmarks.
-        if self < 10 {
-            1
-        } else if self < 100 {
-            2
-        } else {
-            3
-        }
-    }
-}
-impl DigitCount for u16 {
-    fn num_digits(self) -> u8 {
-        // Using a lookup table as with u32 is *not* faster in standalone benchmarks.
-        if self < 10 {
-            1
-        } else if self < 100 {
-            2
-        } else if self < 1_000 {
-            3
-        } else if self < 10_000 {
-            4
-        } else {
-            5
-        }
-    }
-}
-
-impl DigitCount for u32 {
-    fn num_digits(self) -> u8 {
-        /// Lookup table
-        const TABLE: &[u64] = &[
-            0x0001_0000_0000,
-            0x0001_0000_0000,
-            0x0001_0000_0000,
-            0x0001_FFFF_FFF6,
-            0x0002_0000_0000,
-            0x0002_0000_0000,
-            0x0002_FFFF_FF9C,
-            0x0003_0000_0000,
-            0x0003_0000_0000,
-            0x0003_FFFF_FC18,
-            0x0004_0000_0000,
-            0x0004_0000_0000,
-            0x0004_0000_0000,
-            0x0004_FFFF_D8F0,
-            0x0005_0000_0000,
-            0x0005_0000_0000,
-            0x0005_FFFE_7960,
-            0x0006_0000_0000,
-            0x0006_0000_0000,
-            0x0006_FFF0_BDC0,
-            0x0007_0000_0000,
-            0x0007_0000_0000,
-            0x0007_0000_0000,
-            0x0007_FF67_6980,
-            0x0008_0000_0000,
-            0x0008_0000_0000,
-            0x0008_FA0A_1F00,
-            0x0009_0000_0000,
-            0x0009_0000_0000,
-            0x0009_C465_3600,
-            0x000A_0000_0000,
-            0x000A_0000_0000,
-        ];
-        ((self as u64 + TABLE[31_u32.saturating_sub(self.leading_zeros()) as usize]) >> 32) as _
-    }
-}
-// endregion extension trait
-
 /// Write all bytes to the output, returning the number of bytes written.
-fn write(output: &mut impl io::Write, bytes: &[u8]) -> io::Result<usize> {
+pub(crate) fn write(output: &mut impl io::Write, bytes: &[u8]) -> io::Result<usize> {
     output.write_all(bytes)?;
     Ok(bytes.len())
 }
 
+/// If `pred` is true, write all bytes to the output, returning the number of bytes written.
+pub(crate) fn write_if(output: &mut impl io::Write, pred: bool, bytes: &[u8]) -> io::Result<usize> {
+    if pred { write(output, bytes) } else { Ok(0) }
+}
+
+/// If `pred` is true, write `true_bytes` to the output. Otherwise, write `false_bytes`.
+pub(crate) fn write_if_else(
+    output: &mut impl io::Write,
+    pred: bool,
+    true_bytes: &[u8],
+    false_bytes: &[u8],
+) -> io::Result<usize> {
+    write(output, if pred { true_bytes } else { false_bytes })
+}
+
+/// Write the floating point number to the output, returning the number of bytes written.
+///
+/// This method accepts the number of digits before and after the decimal. The value will be padded
+/// with zeroes to the left if necessary.
+pub(crate) fn format_float(
+    output: &mut impl io::Write,
+    value: f64,
+    digits_before_decimal: u8,
+    digits_after_decimal: Option<NonZeroU8>,
+) -> io::Result<usize> {
+    match digits_after_decimal {
+        Some(digits_after_decimal) => {
+            let digits_after_decimal = digits_after_decimal.get() as usize;
+            let width = digits_before_decimal as usize + 1 + digits_after_decimal;
+            write!(output, "{value:0>width$.digits_after_decimal$}")?;
+            Ok(width)
+        }
+        None => {
+            let value = value.trunc() as u64;
+            let width = digits_before_decimal as usize;
+            write!(output, "{value:0>width$}")?;
+            Ok(width)
+        }
+    }
+}
+
 /// Format a number with the provided padding and width.
 ///
 /// The sign must be written by the caller.
-pub(crate) fn format_number<W: io::Write, V: itoa::Integer + DigitCount + Copy, const WIDTH: u8>(
-    output: &mut W,
-    value: V,
+pub(crate) fn format_number<const WIDTH: u8>(
+    output: &mut impl io::Write,
+    value: impl itoa::Integer + DigitCount + Copy,
     padding: modifier::Padding,
 ) -> Result<usize, io::Error> {
     match padding {
-        modifier::Padding::Space => format_number_pad_space::<_, _, WIDTH>(output, value),
-        modifier::Padding::Zero => format_number_pad_zero::<_, _, WIDTH>(output, value),
-        modifier::Padding::None => write(output, itoa::Buffer::new().format(value).as_bytes()),
+        modifier::Padding::Space => format_number_pad_space::<WIDTH>(output, value),
+        modifier::Padding::Zero => format_number_pad_zero::<WIDTH>(output, value),
+        modifier::Padding::None => format_number_pad_none(output, value),
     }
 }
 
 /// Format a number with the provided width and spaces as padding.
 ///
 /// The sign must be written by the caller.
-pub(crate) fn format_number_pad_space<
-    W: io::Write,
-    V: itoa::Integer + DigitCount + Copy,
-    const WIDTH: u8,
->(
-    output: &mut W,
-    value: V,
+pub(crate) fn format_number_pad_space<const WIDTH: u8>(
+    output: &mut impl io::Write,
+    value: impl itoa::Integer + DigitCount + Copy,
 ) -> Result<usize, io::Error> {
     let mut bytes = 0;
     for _ in 0..(WIDTH.saturating_sub(value.num_digits())) {
-        bytes += write(output, &[b' '])?;
+        bytes += write(output, b" ")?;
     }
     bytes += write(output, itoa::Buffer::new().format(value).as_bytes())?;
     Ok(bytes)
@@ -158,22 +118,28 @@ pub(crate) fn format_number_pad_space<
 /// Format a number with the provided width and zeros as padding.
 ///
 /// The sign must be written by the caller.
-pub(crate) fn format_number_pad_zero<
-    W: io::Write,
-    V: itoa::Integer + DigitCount + Copy,
-    const WIDTH: u8,
->(
-    output: &mut W,
-    value: V,
+pub(crate) fn format_number_pad_zero<const WIDTH: u8>(
+    output: &mut impl io::Write,
+    value: impl itoa::Integer + DigitCount + Copy,
 ) -> Result<usize, io::Error> {
     let mut bytes = 0;
     for _ in 0..(WIDTH.saturating_sub(value.num_digits())) {
-        bytes += write(output, &[b'0'])?;
+        bytes += write(output, b"0")?;
     }
     bytes += write(output, itoa::Buffer::new().format(value).as_bytes())?;
     Ok(bytes)
 }
 
+/// Format a number with no padding.
+///
+/// If the sign is mandatory, the sign must be written by the caller.
+pub(crate) fn format_number_pad_none(
+    output: &mut impl io::Write,
+    value: impl itoa::Integer + Copy,
+) -> Result<usize, io::Error> {
+    write(output, itoa::Buffer::new().format(value).as_bytes())
+}
+
 /// Format the provided component into the designated output. An `Err` will be returned if the
 /// component requires information that it does not provide or if the value cannot be output to the
 /// stream.
@@ -200,7 +166,23 @@ pub(crate) fn format_component(
         (OffsetHour(modifier), .., Some(offset)) => fmt_offset_hour(output, offset, modifier)?,
         (OffsetMinute(modifier), .., Some(offset)) => fmt_offset_minute(output, offset, modifier)?,
         (OffsetSecond(modifier), .., Some(offset)) => fmt_offset_second(output, offset, modifier)?,
-        _ => return Err(error::Format::InsufficientTypeInformation),
+        (Ignore(_), ..) => 0,
+        (UnixTimestamp(modifier), Some(date), Some(time), Some(offset)) => {
+            fmt_unix_timestamp(output, date, time, offset, modifier)?
+        }
+        (End(modifier::End {}), ..) => 0,
+
+        // This is functionally the same as a wildcard arm, but it will cause an error if a new
+        // component is added. This is to avoid a bug where a new component, the code compiles, and
+        // formatting fails.
+        // Allow unreachable patterns because some branches may be fully matched above.
+        #[allow(unreachable_patterns)]
+        (
+            Day(_) | Month(_) | Ordinal(_) | Weekday(_) | WeekNumber(_) | Year(_) | Hour(_)
+            | Minute(_) | Period(_) | Second(_) | Subsecond(_) | OffsetHour(_) | OffsetMinute(_)
+            | OffsetSecond(_) | Ignore(_) | UnixTimestamp(_) | End(_),
+            ..,
+        ) => return Err(error::Format::InsufficientTypeInformation),
     })
 }
 
@@ -211,7 +193,7 @@ fn fmt_day(
     date: Date,
     modifier::Day { padding }: modifier::Day,
 ) -> Result<usize, io::Error> {
-    format_number::<_, _, 2>(output, date.day(), padding)
+    format_number::<2>(output, date.day(), padding)
 }
 
 /// Format the month into the designated output.
@@ -225,9 +207,7 @@ fn fmt_month(
     }: modifier::Month,
 ) -> Result<usize, io::Error> {
     match repr {
-        modifier::MonthRepr::Numerical => {
-            format_number::<_, _, 2>(output, date.month() as u8, padding)
-        }
+        modifier::MonthRepr::Numerical => format_number::<2>(output, date.month() as u8, padding),
         modifier::MonthRepr::Long => write(output, MONTH_NAMES[date.month() as usize - 1]),
         modifier::MonthRepr::Short => write(output, &MONTH_NAMES[date.month() as usize - 1][..3]),
     }
@@ -239,7 +219,7 @@ fn fmt_ordinal(
     date: Date,
     modifier::Ordinal { padding }: modifier::Ordinal,
 ) -> Result<usize, io::Error> {
-    format_number::<_, _, 3>(output, date.ordinal(), padding)
+    format_number::<3>(output, date.ordinal(), padding)
 }
 
 /// Format the weekday into the designated output.
@@ -261,12 +241,12 @@ fn fmt_weekday(
             output,
             WEEKDAY_NAMES[date.weekday().number_days_from_monday() as usize],
         ),
-        modifier::WeekdayRepr::Sunday => format_number::<_, _, 1>(
+        modifier::WeekdayRepr::Sunday => format_number::<1>(
             output,
             date.weekday().number_days_from_sunday() + one_indexed as u8,
             modifier::Padding::None,
         ),
-        modifier::WeekdayRepr::Monday => format_number::<_, _, 1>(
+        modifier::WeekdayRepr::Monday => format_number::<1>(
             output,
             date.weekday().number_days_from_monday() + one_indexed as u8,
             modifier::Padding::None,
@@ -280,7 +260,7 @@ fn fmt_week_number(
     date: Date,
     modifier::WeekNumber { padding, repr }: modifier::WeekNumber,
 ) -> Result<usize, io::Error> {
-    format_number::<_, _, 2>(
+    format_number::<2>(
         output,
         match repr {
             modifier::WeekNumberRepr::Iso => date.iso_week(),
@@ -313,18 +293,18 @@ fn fmt_year(
     };
     let format_number = match repr {
         #[cfg(feature = "large-dates")]
-        modifier::YearRepr::Full if value.abs() >= 100_000 => format_number::<_, _, 6>,
+        modifier::YearRepr::Full if value.abs() >= 100_000 => format_number::<6>,
         #[cfg(feature = "large-dates")]
-        modifier::YearRepr::Full if value.abs() >= 10_000 => format_number::<_, _, 5>,
-        modifier::YearRepr::Full => format_number::<_, _, 4>,
-        modifier::YearRepr::LastTwo => format_number::<_, _, 2>,
+        modifier::YearRepr::Full if value.abs() >= 10_000 => format_number::<5>,
+        modifier::YearRepr::Full => format_number::<4>,
+        modifier::YearRepr::LastTwo => format_number::<2>,
     };
     let mut bytes = 0;
     if repr != modifier::YearRepr::LastTwo {
         if full_year < 0 {
-            bytes += write(output, &[b'-'])?;
+            bytes += write(output, b"-")?;
         } else if sign_is_mandatory || cfg!(feature = "large-dates") && full_year >= 10_000 {
-            bytes += write(output, &[b'+'])?;
+            bytes += write(output, b"+")?;
         }
     }
     bytes += format_number(output, value.unsigned_abs(), padding)?;
@@ -348,7 +328,7 @@ fn fmt_hour(
         (hour, true) if hour < 12 => hour,
         (hour, true) => hour - 12,
     };
-    format_number::<_, _, 2>(output, value, padding)
+    format_number::<2>(output, value, padding)
 }
 
 /// Format the minute into the designated output.
@@ -357,7 +337,7 @@ fn fmt_minute(
     time: Time,
     modifier::Minute { padding }: modifier::Minute,
 ) -> Result<usize, io::Error> {
-    format_number::<_, _, 2>(output, time.minute(), padding)
+    format_number::<2>(output, time.minute(), padding)
 }
 
 /// Format the period into the designated output.
@@ -383,7 +363,7 @@ fn fmt_second(
     time: Time,
     modifier::Second { padding }: modifier::Second,
 ) -> Result<usize, io::Error> {
-    format_number::<_, _, 2>(output, time.second(), padding)
+    format_number::<2>(output, time.second(), padding)
 }
 
 /// Format the subsecond into the designated output.
@@ -396,23 +376,23 @@ fn fmt_subsecond<W: io::Write>(
     let nanos = time.nanosecond();
 
     if digits == Nine || (digits == OneOrMore && nanos % 10 != 0) {
-        format_number_pad_zero::<_, _, 9>(output, nanos)
+        format_number_pad_zero::<9>(output, nanos)
     } else if digits == Eight || (digits == OneOrMore && (nanos / 10) % 10 != 0) {
-        format_number_pad_zero::<_, _, 8>(output, nanos / 10)
+        format_number_pad_zero::<8>(output, nanos / 10)
     } else if digits == Seven || (digits == OneOrMore && (nanos / 100) % 10 != 0) {
-        format_number_pad_zero::<_, _, 7>(output, nanos / 100)
+        format_number_pad_zero::<7>(output, nanos / 100)
     } else if digits == Six || (digits == OneOrMore && (nanos / 1_000) % 10 != 0) {
-        format_number_pad_zero::<_, _, 6>(output, nanos / 1_000)
+        format_number_pad_zero::<6>(output, nanos / 1_000)
     } else if digits == Five || (digits == OneOrMore && (nanos / 10_000) % 10 != 0) {
-        format_number_pad_zero::<_, _, 5>(output, nanos / 10_000)
+        format_number_pad_zero::<5>(output, nanos / 10_000)
     } else if digits == Four || (digits == OneOrMore && (nanos / 100_000) % 10 != 0) {
-        format_number_pad_zero::<_, _, 4>(output, nanos / 100_000)
+        format_number_pad_zero::<4>(output, nanos / 100_000)
     } else if digits == Three || (digits == OneOrMore && (nanos / 1_000_000) % 10 != 0) {
-        format_number_pad_zero::<_, _, 3>(output, nanos / 1_000_000)
+        format_number_pad_zero::<3>(output, nanos / 1_000_000)
     } else if digits == Two || (digits == OneOrMore && (nanos / 10_000_000) % 10 != 0) {
-        format_number_pad_zero::<_, _, 2>(output, nanos / 10_000_000)
+        format_number_pad_zero::<2>(output, nanos / 10_000_000)
     } else {
-        format_number_pad_zero::<_, _, 1>(output, nanos / 100_000_000)
+        format_number_pad_zero::<1>(output, nanos / 100_000_000)
     }
 }
 // endregion time formatters
@@ -429,11 +409,11 @@ fn fmt_offset_hour(
 ) -> Result<usize, io::Error> {
     let mut bytes = 0;
     if offset.is_negative() {
-        bytes += write(output, &[b'-'])?;
+        bytes += write(output, b"-")?;
     } else if sign_is_mandatory {
-        bytes += write(output, &[b'+'])?;
+        bytes += write(output, b"+")?;
     }
-    bytes += format_number::<_, _, 2>(output, offset.whole_hours().unsigned_abs(), padding)?;
+    bytes += format_number::<2>(output, offset.whole_hours().unsigned_abs(), padding)?;
     Ok(bytes)
 }
 
@@ -443,7 +423,7 @@ fn fmt_offset_minute(
     offset: UtcOffset,
     modifier::OffsetMinute { padding }: modifier::OffsetMinute,
 ) -> Result<usize, io::Error> {
-    format_number::<_, _, 2>(output, offset.minutes_past_hour().unsigned_abs(), padding)
+    format_number::<2>(output, offset.minutes_past_hour().unsigned_abs(), padding)
 }
 
 /// Format the offset second into the designated output.
@@ -452,6 +432,48 @@ fn fmt_offset_second(
     offset: UtcOffset,
     modifier::OffsetSecond { padding }: modifier::OffsetSecond,
 ) -> Result<usize, io::Error> {
-    format_number::<_, _, 2>(output, offset.seconds_past_minute().unsigned_abs(), padding)
+    format_number::<2>(output, offset.seconds_past_minute().unsigned_abs(), padding)
 }
 // endregion offset formatters
+
+/// Format the Unix timestamp into the designated output.
+fn fmt_unix_timestamp(
+    output: &mut impl io::Write,
+    date: Date,
+    time: Time,
+    offset: UtcOffset,
+    modifier::UnixTimestamp {
+        precision,
+        sign_is_mandatory,
+    }: modifier::UnixTimestamp,
+) -> Result<usize, io::Error> {
+    let date_time = date
+        .with_time(time)
+        .assume_offset(offset)
+        .to_offset(UtcOffset::UTC);
+
+    if date_time < OffsetDateTime::UNIX_EPOCH {
+        write(output, b"-")?;
+    } else if sign_is_mandatory {
+        write(output, b"+")?;
+    }
+
+    match precision {
+        modifier::UnixTimestampPrecision::Second => {
+            format_number_pad_none(output, date_time.unix_timestamp().unsigned_abs())
+        }
+        modifier::UnixTimestampPrecision::Millisecond => format_number_pad_none(
+            output,
+            (date_time.unix_timestamp_nanos() / Nanosecond::per(Millisecond) as i128)
+                .unsigned_abs(),
+        ),
+        modifier::UnixTimestampPrecision::Microsecond => format_number_pad_none(
+            output,
+            (date_time.unix_timestamp_nanos() / Nanosecond::per(Microsecond) as i128)
+                .unsigned_abs(),
+        ),
+        modifier::UnixTimestampPrecision::Nanosecond => {
+            format_number_pad_none(output, date_time.unix_timestamp_nanos().unsigned_abs())
+        }
+    }
+}
